@@ -69,6 +69,7 @@ class DatasetAssembler(EoFields):
         "dtr:end_datetime",
         "dtr:start_datetime",
         "eo:cloud_cover",
+        "eo:constellation",
         "eo:gsd",
         "eo:instrument",
         "eo:platform",
@@ -104,6 +105,7 @@ class DatasetAssembler(EoFields):
         "landsat:landsat_scene_id",
         "landsat:wrs_path",
         "landsat:wrs_row",
+        "mission",
         "odc:region_code",
         "sat:absolute_orbit",
         "sat:anx_datetime",
