@@ -1,3 +1,4 @@
+import re
 from datetime import datetime
 from pathlib import Path
 from typing import Tuple, Dict, Optional, List, Sequence, Union
@@ -109,12 +110,34 @@ class ComplicatedNamingConventions:
         # Is there another organisation you want to use? Pull requests very welcome!
     }
 
+    # The abbreviations mentioned in DEA naming conventions doc.
+    KNOWN_PLATFORM_ABBREVIATIONS = {
+        "landsat-5": "ls5",
+        "landsat-7": "ls7",
+        "landsat-8": "ls8",
+        "landsat-9": "ls9",
+        "sentinel-1a": "s1a",
+        "sentinel-1b": "s1b",
+        "sentinel-2a": "s2a",
+        "sentinel-2b": "s2b",
+        "aqua": "aqu",
+        "terra": "ter",
+    }
+
+    # If all platforms match a pattern, return this group name instead.
+    KNOWN_PLATFORM_GROUPINGS = {
+        "ls": re.compile(r"ls\d+"),
+        "s1": re.compile(r"s1[a-z]+"),
+        "s2": re.compile(r"s2[a-z]+"),
+    }
+
     def __init__(
         self,
         dataset: EoFields,
         base_product_uri: str = None,
         required_fields: Sequence[str] = (),
         dataset_separator_field: Optional[str] = None,
+        allow_unknown_abbreviations: bool = True,
     ) -> None:
         self.dataset = dataset
         self.base_product_uri = base_product_uri
@@ -126,27 +149,35 @@ class ComplicatedNamingConventions:
         if self.dataset_separator_field is not None:
             self.required_fields.add(dataset_separator_field)
 
+        self.allow_unknown_abbreviations = allow_unknown_abbreviations
+
     @classmethod
     def for_standard_dea(cls, dataset: EoFields, uri=DEA_URI_PREFIX):
         """
         Strict mode to follow the full DEA naming conventions.
 
         Only use the (default) DEA URI if you're making DEA products.
+
+        Example file structure (note version number in file):
+            ga_ls8c_ones_3/090/084/2016/01/21/ga_ls8c_ones_3-0-0_090084_2016-01-21_final.odc-metadata.yaml
+
+
         """
         return cls(
             dataset=dataset,
             base_product_uri=uri,
             # These fields are needed to fulfill official DEA naming conventions.
             required_fields=(
-                # TODO: Add conventions for multi-platform/composite products?
-                "eo:instrument",
                 "eo:platform",
-                "odc:dataset_version",
+                "eo:instrument",
                 "odc:processing_datetime",
                 "odc:producer",
                 "odc:product_family",
                 "odc:region_code",
+                "odc:dataset_version",
             ),
+            # DEA wants consistency via the naming-conventions doc.
+            allow_unknown_abbreviations=False,
         )
 
     @classmethod
@@ -154,7 +185,7 @@ class ComplicatedNamingConventions:
         """
         DEA naming conventions, but with an extra subfolder for each unique datatake.
 
-        It will figure out the datatake if you set a sentinel_tile_id.
+        It will figure out the datatake if you set a sentinel_tile_id or datastrip_id.
         """
         return cls(
             dataset=dataset,
@@ -170,6 +201,8 @@ class ComplicatedNamingConventions:
                 "odc:region_code",
             ),
             dataset_separator_field="sentinel:datatake_start_datetime",
+            # DEA wants consistency via the naming-conventions doc.
+            allow_unknown_abbreviations=False,
         )
 
     def _check_enough_properties_to_name(self):
@@ -195,6 +228,9 @@ class ComplicatedNamingConventions:
 
     @property
     def product_name(self) -> str:
+        if self.dataset.product_name:
+            return self.dataset.product_name
+
         self._check_enough_properties_to_name()
 
         org_number = self._org_collection_number
@@ -204,36 +240,49 @@ class ComplicatedNamingConventions:
 
     @property
     def _org_collection_number(self) -> Optional[int]:
+        # An explicit collection number trumps all.
+        if self.dataset.collection_number:
+            return int(self.dataset.collection_number)
+
+        # Otherwise it's the first digit of the dataset version.
         if not self.dataset.dataset_version:
             return None
         return int(self.dataset.dataset_version.split(".")[0])
 
     def _product_group(self, subname=None) -> str:
-        # Fallback to the whole product's name
-        if not subname:
-            subname = self.dataset.product_family
-
         parts = []
-        if self.producer_abbreviated:
-            parts.append(self.producer_abbreviated)
 
-        platform = self.platform_abbreviated
-        inst = self.instrument_abbreviated
-        if platform and inst:
-            parts.append(f"{platform}{inst}")
+        # If they've given a product name, just use it.
+        if self.dataset.product_name:
+            parts.append(self.dataset.product_name)
+            if subname:
+                parts.append(subname)
+        else:
+            self._check_enough_properties_to_name()
 
-        if not subname:
-            raise ValueError(
-                "Not even metadata to create a useful filename! "
-                'Set the `product_family` (eg. "wofs") or a subname'
-            )
-        parts.append(subname)
+            # They're not specifying a sub-file. Fallback to the whole product's category.
+            if not subname:
+                subname = self.dataset.product_family
+
+            if self.producer_abbreviated:
+                parts.append(self.producer_abbreviated)
+
+            platform = self.platform_abbreviated
+            inst = self.instrument_abbreviated or ""
+            if platform:
+                parts.append(f"{platform}{inst}")
+
+            if not subname:
+                raise ValueError(
+                    "Not enough metadata to create a useful filename! "
+                    'Set the `product_family` (eg. "wofs") or a subname'
+                )
+            parts.append(subname)
 
         return "_".join(parts)
 
     @property
     def product_uri(self) -> Optional[str]:
-        self._check_enough_properties_to_name()
         if not self.base_product_uri:
             return None
 
@@ -244,11 +293,9 @@ class ComplicatedNamingConventions:
         """
         Label for a dataset
         """
-        self._check_enough_properties_to_name()
         return self._dataset_label()
 
     def destination_folder(self, base: Path) -> Path:
-        self._check_enough_properties_to_name()
         # DEA naming conventions folder hierarchy.
         # Example: "ga_ls8c_ard_3/092/084/2016/06/28"
 
@@ -275,17 +322,14 @@ class ComplicatedNamingConventions:
         return base.joinpath(*parts)
 
     def metadata_path(self, work_dir: Path, kind: str = "", suffix: str = "yaml"):
-        self._check_enough_properties_to_name()
         return self._file(work_dir, kind, suffix)
 
     def checksum_path(self, work_dir: Path, suffix: str = "sha1"):
-        self._check_enough_properties_to_name()
         return self._file(work_dir, "", suffix)
 
     def measurement_file_path(
         self, work_dir: Path, measurement_name: str, suffix: str, file_id: str = None
     ) -> Path:
-        self._check_enough_properties_to_name()
         if ":" in measurement_name:
             subgroup, name = measurement_name.split(":")
         else:
@@ -340,53 +384,75 @@ class ComplicatedNamingConventions:
     @property
     def platform_abbreviated(self) -> Optional[str]:
         """Abbreviated form of a satellite, as used in dea product names. eg. 'ls7'."""
-        p = self.dataset.platform
+
+        p = self.dataset.platforms
         if not p:
             return None
 
-        if p.startswith("sentinel-2"):
-            return f"s2{p[-1]}"
+        if not self.allow_unknown_abbreviations:
+            unknowns = p.difference(self.KNOWN_PLATFORM_ABBREVIATIONS)
+            if unknowns:
+                raise ValueError(
+                    f"We don't know the DEA abbreviation for platforms {unknowns!r}. "
+                    f"We'd love to add more! Raise an issue on Github: "
+                    f"https://github.com/GeoscienceAustralia/eo-datasets/issues/new' "
+                )
 
-        if p.startswith("sentinel-1"):
-            return f"s1{p[-1]}"
+        abbreviations = sorted(
+            self.KNOWN_PLATFORM_ABBREVIATIONS.get(s, s.replace("-", "")) for s in p
+        )
 
-        if not p.startswith("landsat"):
-            raise NotImplementedError(
-                f"TODO: implement non-landsat platform abbreviation " f"(got {p!r})"
-            )
+        if len(abbreviations) == 1:
+            return abbreviations[0]
 
-        return f"ls{p[-1]}"
+        # If all abbreviations are in a group, name it using that group.
+        # (eg. "ls" instead of "ls5-ls7-ls8")
+        for group_name, pattern in self.KNOWN_PLATFORM_GROUPINGS.items():
+            if all(pattern.match(a) for a in abbreviations):
+                return group_name
+
+        # Otherwise, there's a mix of platforms.
+
+        # Is there a common constellation?
+        constellation = self.dataset.properties.get("constellation")
+        if constellation:
+            return constellation
+
+        # Don't bother to include platform in name for un-groupable mixes of them.
+        return None
 
     @property
     def instrument_abbreviated(self) -> Optional[str]:
         """Abbreviated form of an instrument name, as used in dea product names. eg. 'c'."""
-        p = self.dataset.platform
-        if not p:
+        platforms = self.dataset.platforms
+        if not platforms or len(platforms) > 1:
             return None
 
-        if p.startswith("sentinel-2"):
+        [p] = platforms
+
+        if p.startswith("sentinel-1") or p.startswith("sentinel-2"):
             return self.dataset.instrument[0].lower()
 
-        if p.startswith("sentinel-1"):
-            return self.dataset.instrument[0].lower()
+        if p.startswith("landsat"):
+            # Extract from usgs standard:
+            # landsat:landsat_product_id: LC08_L1TP_091075_20161213_20170316_01_T2
+            # landsat:landsat_scene_id: LC80910752016348LGN01
+            landsat_id: str = self.dataset.properties.get(
+                "landsat:landsat_product_id"
+            ) or self.dataset.properties.get("landsat:landsat_scene_id")
+            if not landsat_id:
+                raise NotImplementedError(
+                    "TODO: Can only currently abbreviate instruments from Landsat references."
+                )
 
-        if not p.startswith("landsat"):
-            raise NotImplementedError(
-                f"TODO: implement non-landsat instrument abbreviation " f"(got {p!r})"
-            )
+            return landsat_id[1].lower()
 
-        # Extract from usgs standard:
-        # landsat:landsat_product_id: LC08_L1TP_091075_20161213_20170316_01_T2
-        # landsat:landsat_scene_id: LC80910752016348LGN01
-        landsat_id: str = self.dataset.properties.get(
-            "landsat:landsat_product_id"
-        ) or self.dataset.properties.get("landsat:landsat_scene_id")
-        if not landsat_id:
-            raise NotImplementedError(
-                "TODO: Can only currently abbreviate instruments from Landsat references."
-            )
-
-        return landsat_id[1].lower()
+        # Otherwise, it's unknown.
+        raise NotImplementedError(
+            f"Instrument abbreviations aren't supported for platform {p!r}. "
+            f"We'd love to add more support! Raise an issue on Github: "
+            f"https://github.com/GeoscienceAustralia/eo-datasets/issues/new' "
+        )
 
     @property
     def producer_abbreviated(self) -> Optional[str]:
@@ -406,14 +472,31 @@ class ComplicatedNamingConventions:
 
 class ComplicatedNamingConventionsDerivatives(ComplicatedNamingConventions):
     """
-    This class is inherited from ComplicatedNamingConventions
-    and overrides few attributes specific to C3 data processing for data other than ARD.
+    Derivatives have a slightly different folder structure.
+
+    And they only show constellations (eg. "ls_" or "s2_") rather than the specific
+    satellites in their names (eg. "ls8_").
+
+    They have a version-number folder instead of putting it in each filename.
+
+    And version numbers may not match the collection number (`odc:collection_number` is
+    mandatory).
     """
 
     @classmethod
     def for_c3_derivatives(cls, dataset: EoFields, uri=DEA_URI_PREFIX):
         """
-        The required fields for the c3 data processing are controlled here.
+        Create naming conventions for common derived products.
+
+        Unlike plain 'DEA', they use an explicit collection number (odc:collection_number)
+        in the product name which may differ from the software's dataset version
+        (odc:dataset_version)
+
+        Example file structure (note version number in folder):
+
+            ga_ls_wo_3/1-6-0/090/081/1998/07/30/ga_ls_wo_3_090081_1998-07-30_interim.odc-metadata.yaml
+
+
         """
         return cls(
             dataset=dataset,
@@ -426,6 +509,7 @@ class ComplicatedNamingConventionsDerivatives(ComplicatedNamingConventions):
                 "odc:producer",
                 "odc:product_family",
                 "odc:region_code",
+                "odc:dataset_version",
                 "dea:dataset_maturity",
             ),
         )
@@ -433,7 +517,11 @@ class ComplicatedNamingConventionsDerivatives(ComplicatedNamingConventions):
     @classmethod
     def for_s2_derivatives(cls, dataset: EoFields, uri=DEA_URI_PREFIX):
         """
-        The required fields for the S2 data processing are controlled here.
+        DEA derivative naming conventions, but with an extra subfolder for
+        each unique datatake.
+
+        It will figure out the datatake if you set a sentinel_tile_id
+        or datastrip_id.
         """
         return cls(
             dataset=dataset,
@@ -447,16 +535,12 @@ class ComplicatedNamingConventionsDerivatives(ComplicatedNamingConventions):
                 "odc:producer",
                 "odc:product_family",
                 "odc:region_code",
+                "odc:dataset_version",
                 "dea:dataset_maturity",
             ),
             dataset_separator_field="sentinel:datatake_start_datetime",
         )
 
-    @property
-    def _org_collection_number(self) -> Optional[int]:
-        # Deliberately fail if collection_number is not defined in the config yaml
-        return int(self.dataset.collection_number)
-
     def _product_group(self, subname=None) -> str:
         # Computes product group, e.g "ga_ls_wo_3"
         # Deliberately fail if any of these attributes not found.
@@ -499,17 +583,27 @@ class ComplicatedNamingConventionsDerivatives(ComplicatedNamingConventions):
 
     @property
     def platform_abbreviated(self) -> Optional[str]:
-        # For now from Alchemist the platform is always landsat for C3 processing
-        if "landsat" in self.dataset.platform:
-            return "ls"
+        """
+        Derivatives only show group/constellation names ("ls" or "s2". Not "ls8".)
 
-        if "sentinel-2" in self.dataset.platform:
-            return "s2"
+        (Because individual datasets come from different platforms, a specific abbreviation
+        would be inconsistent across a product [presumably])
+        """
+        abbreviations = sorted(
+            self.KNOWN_PLATFORM_ABBREVIATIONS.get(s, s.replace("-", ""))
+            for s in self.dataset.platforms
+        )
+        # If all abbreviations are in a group, name it using that group.
+        # (eg. "ls" instead of "ls5-ls7-ls8")
+        for group_name, pattern in self.KNOWN_PLATFORM_GROUPINGS.items():
+            if all(pattern.match(a) for a in abbreviations):
+                return group_name
 
         raise NotImplementedError(
-            f"Only Landsat and Sentinel 2 platforms currently implemented for dea_c3 "
-            f"and dea_s2_derivative naming schemes "
-            f"(got {self.dataset.platform!r})"
+            f"Satellite constellation abbreviation is not known for platforms {self.dataset.platforms}. "
+            f"(for DEA derivative naming conventions.)"
+            f"    Is this a mistake? We'd love to add more! Raise an issue on Github: "
+            f"https://github.com/GeoscienceAustralia/eo-datasets/issues/new' "
         )
 
 
